require 'terminal-table'
require 'colorize'
require 'launchy'

module Kickstand
  class CLI

    attr_reader :stdout

    def initialize(stdout: $stdout)
      @stdout = stdout
    end

    def restart(id)
      workflow = Kickstand::Workflow.find(id)
      workflow.mark_as_started
      workflow.save!
      EnqueuerWorker.set(queue: configuration.namespace).perform_later(workflow.id.to_s)
    end

    def stop(id)
      workflow = Kickstand::Workflow.find(id)
      workflow.mark_as_stopped
      workflow.save!
    end

    def show(id)
      workflow = Kickstand::Workflow.find(id)
      display_overview_for(workflow)
      display_jobs_list_for(workflow)
    end

    def rm(workflow_id)
      workflow = Kickstand::Workflow.find(workflow_id)
      workflow.destroy!
    end
    def list
      workflows = Kickstand::Workflow.all
      rows = []

      workflows.map do |workflow|
        rows << [workflow.id, (Time.at(workflow.started_at) if workflow.started_at), workflow.class, {alignment: :center, value: status_for(workflow)}]
      end

      headers = [
        {alignment: :center, value: 'id'},
        {alignment: :center, value: 'started at'},
        {alignment: :center, value: 'name'},
        {alignment: :center, value: 'status'}
      ]
      stdout.puts Terminal::Table.new(headings: headers, rows: rows)
    end

    private

    def overview(workflow)
      CLI::Overview.new(workflow)
    end

    def display_overview_for(workflow)
      stdout.puts overview(workflow).table
    end

    def status_for(workflow)
      overview(workflow).status
    end

    def display_jobs_list_for(workflow)
      stdout.puts overview(workflow).jobs_list(workflow.jobs)
    end

  end
end
