require 'redis'
require 'concurrent-ruby'

module Kickstand
  class RedisClient
    attr_reader :configuration

    EnqueuerAlreadyBeingProcessed = Class.new(StandardError)

    @@redis_connection = Concurrent::ThreadLocalVar.new(nil)

    def self.redis_connection
      config = Kickstand.configuration
      cached = (@@redis_connection.value ||= config.redis_opts.merge({ connection: nil }))
      return cached[:connection] if !cached[:connection].nil? && config.redis_opts[:url] == cached[:url]
      
      Redis.new(config.redis_opts).tap do |instance|
        RedisClassy.redis = instance
        @@redis_connection.value = config.redis_opts.merge({ connection: instance })
      end
    end

    def secure_enqueuer_locker(workflow_id, duration)
      raise EnqueuerAlreadyBeingProcessed unless redis.set("kickstand.locker.#{workflow_id.to_s}", 1, :nx => true, :ex => duration.seconds)
    end

    def release_enqueuer_locker(workflow_id)
      redis.del("kickstand.locker.#{workflow_id.to_s}")
    end

    private

    def redis
      self.class.redis_connection
    end
  end
end
