module Kickstand
  class Configuration
    attr_accessor :namespace, :redis_opts, :enqueuer_frequency,
                  :on_job_failure, :on_workflow_failure, :on_job_retry, :on_job_timeout, :on_job_lost, :on_job_enqueue,
                  :on_workflow_persist, :on_workflow_start, :on_workflow_finish, :on_workflow_auto_destroy

    def self.from_json(json)
      new(Kickstand::JSON.decode(json, symbolize_keys: true))
    end

    def initialize(hash = {})
      self.namespace = hash.fetch(:namespace, 'kickstand')
      self.redis_opts = hash.fetch(:redis_opts, { url: 'redis://localhost:6379', :connect_timeout => 0.5, :read_timeout => 1, :write_timeout => 2 })
      self.kickstandfile = hash.fetch(:kickstandfile, 'Kickstand')
      self.enqueuer_frequency = hash.fetch(:enqueuer_frequency, 30.seconds)
      self.on_job_failure = hash.fetch(:on_job_failure, nil)
      self.on_job_retry = hash.fetch(:on_job_retry, nil)
      self.on_job_timeout = hash.fetch(:on_job_timeout, nil)
      self.on_job_lost = hash.fetch(:on_job_lost, nil)
      self.on_job_enqueue = hash.fetch(:on_job_enqueue, nil)
      self.on_workflow_persist = hash.fetch(:on_workflow_persist, nil)
      self.on_workflow_failure = hash.fetch(:on_workflow_failure, nil)
      self.on_workflow_start = hash.fetch(:on_workflow_start, nil)
      self.on_workflow_finish = hash.fetch(:on_workflow_finish, nil)
      self.on_workflow_auto_destroy = hash.fetch(:on_workflow_auto_destroy, nil)
    end

    def kickstandfile=(path)
      @kickstandfile = Pathname(path)
    end

    def kickstandfile
      @kickstandfile.realpath if @kickstandfile.exist?
    end

    def to_hash
      {
        namespace:            namespace,
        redis_opts:           redis_opts,
        enqueuer_frequency: enqueuer_frequency
      }
    end

    def to_json
      Kickstand::JSON.encode(to_hash)
    end
  end
end
