require 'active_job'

module Kickstand
  class Worker < ::ActiveJob::Base

    delegate :payloads, to: :job

    def perform(workflow_id, job_id)
      setup_job(workflow_id, job_id)
      return unless job.present?
      return if job.succeeded? || job.failed? || job.running?

      mark_as_started
      begin
        job.perform
      rescue Exception => error
        mark_as_failed
        Kickstand.configuration.on_job_failure.call(job, error) if Kickstand.configuration.on_job_failure
      else
        mark_as_finished
      end
    end

    private

    attr_reader :workflow_id, :job

    def setup_job(workflow_id, job_id)
      @workflow_id = workflow_id
      @job ||= Kickstand::Job.where(workflow_id: workflow_id, id: job_id).first
    end

    def mark_as_finished
      if job.output_payload.present?
        job.set(finished_at: Time.now, output_payload: job.output_payload, status: "succeeded")
      else
        job.set(finished_at: Time.now, status: "succeeded")
      end
    end

    def mark_as_failed
      job.set(failed_at: Time.now, status: "failed")
    end

    def mark_as_started
      job.set(started_at: Time.now, number_of_executions: job.number_of_executions + 1, status: "running")
    end

  end
end
