require 'spec_helper'

describe Kickstand::DelayedWorkflowCreatorWorker do
  let(:job) { described_class.new }

  describe "#perform" do
    let(:starts_at) { nil }

    subject(:perform) { job.perform("Kickstand::Workflow", { foo: :bar }.to_json, 10, 5, false, starts_at) }

    context "when the workflow is not scheduled" do
      let(:starts_at) { nil }

      it "creates a workflow and starts it" do
        expect { perform }.to change { Kickstand::Workflow.count }.by(1)
        expect(Kickstand::Workflow.last.started?).to be_truthy
      end
    end

    context "when the workflow is scheduled" do
      let(:starts_at) { 1.hour.from_now }

      context "and it is overdue" do
        let(:starts_at) { 1.hour.ago }

        it "creates a workflow and let clockwork starts it" do
          expect { perform }.to change { Kickstand::Workflow.count }.by(1)
          expect(Kickstand::Workflow.last.started?).to be_falsy
        end
      end

      context "and it is not overdue" do
        it "creates a workflow and starts it" do
          expect { perform }.to change { Kickstand::Workflow.count }.by(1)
          expect(Kickstand::Workflow.last.started?).to be_falsy
        end
      end
    end
  end
end
