require 'spec_helper'

describe Kickstand::Job do

  describe "#output" do
    it "saves output to output_payload" do
      job = described_class.new()
      job.output({ "something" => true })
      expect(job.output_payload).to eq({ "something" => true })
    end
  end
  describe "#fail!" do
    it "sets finished and failed to true and records time" do
      job = described_class.new()
      job.fail!
      expect(job.failed_at).to eq(Time.now.to_i)
      expect(job.failed?).to eq(true)
      expect(job.finished?).to eq(true)
      expect(job.running?).to eq(false)
      expect(job.enqueued?).to eq(false)
    end
  end

  describe "#finish!" do
    it "sets finished to false and failed to false and records time" do
      job = described_class.new()
      job.finish!
      expect(job.finished_at).to eq(Time.now.to_i)
      expect(job.failed?).to eq(false)
      expect(job.running?).to eq(false)
      expect(job.finished?).to eq(true)
      expect(job.enqueued?).to eq(false)
    end
  end

  describe "#enqueue!" do
    it "resets flags to false and sets enqueued to true" do
      job = described_class.new()
      job.finished_at = 123
      job.failed_at = 123
      job.enqueue!
      expect(job.started_at).to eq(nil)
      expect(job.finished_at).to eq(nil)
      expect(job.failed_at).to eq(nil)
      expect(job.failed?).to eq(false)
      expect(job.finished?).to eq(false)
      expect(job.enqueued?).to eq(true)
      expect(job.running?).to eq(false)
    end
  end

  describe "#start!" do
    it "resets flags and marks as running" do
      job = described_class.new()

      job.enqueue!
      job.fail!

      now = Time.now.to_i
      expect(job.started_at).to eq(nil)
      expect(job.failed_at).to eq(now)

      job.start!

      expect(job.started_at).to eq(Time.now.to_i)
      expect(job.failed_at).to eq(nil)
    end
  end

  describe "#timed_out" do
    it "returns false if job failed" do
      job = described_class.new()

      job.enqueue!
      job.start!
      job.fail!

      expect(job.timed_out?).to eq(false)
    end

    it "returns false if job didn't start" do
      job = described_class.new()

      job.enqueue!

      expect(job.timed_out?).to eq(false)
    end

    it "returns false if job finished" do
      job = described_class.new()

      job.enqueue!
      job.start!
      job.finish!

      expect(job.timed_out?).to eq(false)
    end

    it "returns true if job started and didn't finish inside the timeout window" do
      job = described_class.new(timeout: 1)

      job.enqueue!
      job.start!
      sleep 2
      expect(job.timed_out?).to eq(true)
    end

    it "returns true if job started and didn't finish" do
      job = described_class.new(timeout: 300)

      job.enqueue!
      job.start!
      sleep 2
      expect(job.timed_out?).to eq(false)
    end
  end

end
