require 'spec_helper'

describe Kickstand::Worker do
  subject { described_class.new }

  let!(:workflow)   { TestWorkflow.create }
  let!(:job)        { workflow.jobs.first  }
  let(:config) { Kickstand.configuration.to_json  }
  let!(:client)     { Kickstand::RedisClient.new }

  describe "#perform" do
    context "when job fails" do

      class FailingJob < Kickstand::Job
        def perform
          invalid.code_to_raise.error
        end
      end

      class FailingWorkflow < Kickstand::Workflow
        def configure
          run FailingJob
        end
      end

      it "should fail the job and retry again" do
        workflow = FailingWorkflow.create
        job = workflow.find_job("FailingJob")
        subject.perform(workflow.id, job.id)
        job = workflow.find_job("FailingJob")
        subject.perform(workflow.id, job.id)
        job = workflow.find_job("FailingJob")
        subject.perform(workflow.id, job.id)
        expect(job.failed?).to eq(true)
      end
    end

    context "when job completes successfully" do
      it "should mark it as succedeed" do
        expect(subject).to receive(:mark_as_finished)
        job = workflow.find_job("Prepare")
        subject.perform(workflow.id, job.id)
      end
    end

    context "when job changes somehow" do
      it "should update updated_at" do
        expect(subject).to receive(:mark_as_finished)
        job = workflow.find_job("Prepare")
        expect(job.updated_at).to eq(nil)
        subject.perform(workflow.id, job.id)
        expect(job.reload.updated_at).not_to eq(nil)
      end
    end

    it "calls job.perform method" do
      SPY = double()
      expect(SPY).to receive(:some_method)

      class OkayJob < Kickstand::Job
        def perform
          SPY.some_method
        end
      end

      class OkayWorkflow < Kickstand::Workflow
        def configure
          run OkayJob
        end
      end

      workflow = OkayWorkflow.create
      job = workflow.find_job('OkayJob')

      subject.perform(workflow.id, job.id)
    end
  end
end
